% Follow the Cross validation Framework
function [cv_res time margin_cv metadata] = CV_SVMRFE_CVNumFeat(datapath)
%% SVMRFE 
% inner loop for both SVMRFE and number of optimum features.

%% Copyright Huyen Do 2012


X = dlmread([datapath '.data']);
 
[numInstances nCol] = size(X);

Y = X(1:numInstances, end);
X = X(1:numInstances, 1:end-1);

load('randomSeed.mat')
  
vectorC = [ 0.1 1 10 100 1000];

lambdareg = 1e-10; 
kfold_outer = 10; %% 10 fold cross validations
kfold_inner = 10;%% 10 fold inner cross validations

OptCriterion = 'error';

% standadize the data % always do this!
X = normalizemeanstd(X);

% Normalize the data (kernel)
for i = 1:numInstances
	X(i,: ) = X(i,: ) / norm(X(i,: ));
end

%For time counting
time = 0;
option.verbose = 1;

metadata.vectorC    = vectorC;
metadata.randomSeed = randomSeed;  

cv_res.OptCriterion = OptCriterion; 

[numInstances,numFeatures]   = size(X);

metadata.numFeatures    = numFeatures;
metadata.numInstances   = numInstances;
metadata.class1         = sum(Y == 1);
metadata.class2         = sum(Y==-1);
metadata.method         = 'SVMRFE-innerCV-chooseNumFeat';

cv_res.Ytest         = cell(kfold_outer,1);
cv_res.alg           = cell(kfold_outer,1);

cv_res.optimalC         = zeros(kfold_outer,1);

cv_res.averageErr       = 0;
cv_res.averageF1  = 0; 
cv_res.averagePrecision = 0 ; 
cv_res.averageRecall  = 0;  

YPredict_outer = ones(length(Y),1);


%%case1:most expensive, try all number of features
%vectorNum =  [1: length(rankedFeat) ];
%%case2: less expensive, try 1%, 2%, 3%, ... 100% features

numEliminate = max(floor(size(X,2)/100), 1); % 1% number of feature is eliminated in SVMRFE

tmpvect = 1:100; 

vectorNum =  [ 1 floor(tmpvect*size(X,2)/100) size(X,2)];

metadata.vectorNumF    = vectorNum;
metadata.numEliminate  = numEliminate; 


%% Do kfold_outer cross-validation. Inside each fold, do kfold_inner cross validation
% again to choose the optimal value of C
% 
% Save the current state, generate 100 values, reset the state, and repeat the sequence:
% s = randn('state');
% u1 = randn(100);
% randn('state',s);
% u2 = randn(100);      % Contains exactly the same values as u1.

% randomize X and Y
rand('twister',randomSeed);
mixedIndice = randperm(numInstances);

numOuterTest = floor(numInstances/kfold_outer);

tic; % time counting

for cv = 1 : kfold_outer
   fprintf('CV_SVMRFE_CV_OptNumFeat Cross validation outer CV %g\n',cv);
   if(cv < kfold_outer)
        outer.test   = mixedIndice(((cv-1)*numOuterTest+1):(cv*numOuterTest));
        outer.train  = setdiff(mixedIndice, outer.test);
   else
        outer.test   = mixedIndice(((cv-1)*numOuterTest+1):end);
        outer.train  = setdiff(mixedIndice, outer.test);
   end
   numInnerTest = floor(length(outer.train)/kfold_inner);

    %10 fold cross validation to choose optimal C
    %mix again the globalTrain
    rand('twister',randomSeed);
    randOrder   = randperm(length(outer.train));
    outer.train = outer.train(randOrder);
   
    inner.margin = zeros(kfold_inner,length(vectorC),length(vectorNum));
    inner.error  = zeros(kfold_inner,length(vectorC),length(vectorNum));
    
    Ypredict_inner = zeros(length(outer.train),length(vectorC), length(vectorNum));
    
    %% SVMRFE to rank feature, the parameter C is fixed to 1. 
        %fprintf('running inner SVMRFE-all-err Cross validation  outer CV = %g,  inner CV =  %g \n',cv, i);
        rankedFeat = SVMRFE(X(outer.train, :), Y(outer.train), 1, numEliminate, lambdareg); 
        X_tmp = X(:, rankedFeat); 
        
    for i = 1 : kfold_inner
        fprintf('CV_SVMRFE_CV_OptNumFeat Cross validation  outer CV = %g,  inner CV =  %g \n',cv, i);
        %index of test and training samples
        
        index = [];
        if(i < kfold_inner)
            index = ((i-1)*numInnerTest+1):(i*numInnerTest);           
        else
            index = [((i-1)*numInnerTest+1):length(outer.train)];            
        end

        inner.test  = outer.train(index);
        inner.train = setdiff(outer.train, inner.test);
        
        
        % Repeat for different value of C
        for j =  1:length(vectorC)
            C = vectorC(j);
            % Repeat for different value of B
            for k = 1:length(vectorNum)
                numF = vectorNum(k);
                % SVMRFE-all-err  ==============================
                
                [alpha, alpha_sv, obj_value, w_sqd, b, pos, train_err] = SVM(X_tmp(inner.train,1:numF), Y(inner.train), X_tmp(inner.train,1:numF) * X_tmp(inner.train,1:numF)', lambdareg, C,[]);
                Ktest = X_tmp(inner.test,1:numF) * X_tmp(inner.train(pos),1:numF)';
                y_predict =Ktest  * alpha_sv  + b; 
                
                accuracy    = mean(sign(y_predict) == Y(inner.test));
                Ypredict_inner(index,j,k) = sign(y_predict);
                
                inner.error(i,j,k)  = 1 - accuracy;
                inner.margin(i,j,k) = w_sqd;
               
                % end SVMRFE-all-err  =============================                           
            end

        end
    end; % end inner cross validation

    %precision_inner= zeros(length(vectorC),1);
    %recall_inner   = zeros(length(vectorC),1);
    F1score_inner  = zeros(length(vectorNum),length(vectorC));
    err_inner      = zeros(length(vectorNum),length(vectorC));
    
    for i = 1: length(vectorNum) % for a fix B
        for j = 1:length(vectorC) % for a fix C
            [F1score_inner(i,j) precision_inner recall_inner] = F1score(Ypredict_inner(:,j,i), Y(outer.train));
        end
        err_inner(i, :) = mean(Ypredict_inner(:,:,i) ~= repmat(Y(outer.train), 1, length(vectorC)));  % mean error by column or mean by each C

    end
    
    %err_inner and F1score_inner are matrices with columns corresponding to
    % C and rows corresponding to Cr
    
   
    if (strcmp(OptCriterion, 'error'))
        [indexOpt] =  find( err_inner == min(err_inner(:))) ;  % linear indexing %%% MIN Error
        
    elseif(strcmp(OptCriterion, 'F1score'))
        [indexOpt] =  find( F1score_inner == max(F1score_inner(:))) ;  % linear indexing  %%% MAX F1 score
    end
    
    optNumIndex = [];
    optCIndex  = []; 
    for i = 1: length(indexOpt)
        [optNumIndex(i) optCIndex(i)] = ind2sub(size(err_inner), indexOpt(i)); 
    end
   

    % now compute with optimal C and optimal number of top features

    C       = vectorC(optCIndex(1)); 
    NumF    = vectorNum(optNumIndex(1)); 
   
    
    [alpha, alpha_sv, obj_value, w_sqd, b, pos, train_err] = SVM(X_tmp(outer.train,1:NumF), Y(outer.train), X_tmp(outer.train,1:NumF)*X_tmp(outer.train,1:NumF)', lambdareg, C,[]);  
    Ktest = X_tmp(outer.test,1:NumF)*X_tmp(outer.train(pos),1:NumF)';
    y_predict = Ktest * alpha_sv  + b; 
    
    accuracy  = mean(sign(y_predict) == Y(outer.test));
    [F1 precision recall]             = F1score(sign(y_predict), Y(outer.test));
   
    C_opt = C; 
    NumF_opt = NumF;
   fprintf('CV_SVMRFE_CV_OptNumFeat cross validation  = %g, optimalC = %g, optC_r = %g, error %f\n',cv, C_opt, NumF_opt, 1-accuracy );

    % end SVMRFE===========================
    
    result.error_test =   1 -accuracy;
    result.y_predict  =  y_predict;
    result.alpha          = alpha; 
    result.b          = b; 
    
        
    result.F1score      = F1;
    result.precision    = precision; 
    result.recall       = recall; 
    

    cv_res.alg{cv,1}    = result;
    cv_res.Ytest{cv,1}  = Y(outer.test);
   
    cv_res.optimalC(cv)     = C_opt;
    cv_res.optimalNumF(cv)    = NumF_opt;
    cv_res.allOptC{cv,1}    = vectorC(optCIndex); 
    cv_res.allOptNumF{cv,1}   = vectorNum(optNumIndex);
    
    YPredict_outer(outer.test) = sign(result.y_predict);
    
    cv_res.inner_err{cv, 1}   = inner.error;
    cv_res.inner_margin{cv,1} = inner.margin;

    cv_res.averageF1        = cv_res.averageF1 + F1;
    cv_res.averagePrecision = cv_res.averagePrecision + precision;
    cv_res.averageRecall    = cv_res.averageRecall + recall; 
end % end outer crossvalidation

time = toc;
cv_res.YPrediction      = YPredict_outer;
cv_res.averageErr       = sum(YPredict_outer ~= Y)/length(Y);


cv_res.averageF1        = cv_res.averageF1 / kfold_outer; 
cv_res.averagePrecision  = cv_res.averagePrecision / kfold_outer; 
cv_res.averageRecall     = cv_res.averageRecall / kfold_outer; 

cv_res.datapath = datapath;
cv_res.rankedFeat = rankedFeat;

save([folder datapath  '.CV_SVMRFE_CV_OptNumFeat.lambreg' num2str(lambdareg)  '.result.mat'], 'cv_res');

