function [mu, Alpha_sv, b, pos, CostNew] = RSVMPlusL1Update(basicKernels, currentMu, pos, Alpha_sv,b,C, C_r,Y, currentGrad, currentCost, componentDistances, option)

%%
% Extend from Simple MKL - Bach 2009
% Copyright@Huyen Do 2012
%%

%------------------------------------------------------------------------------%
% Initialize
%------------------------------------------------------------------------------%

gold    = (sqrt(5)+1)/2 ;
muInit  = currentMu;
muNew   = muInit ; 

%---------------------------------------------------------------
% Compute Current Cost and Gradient
%%--------------------------------------------------------------
%       %  CostNew = SVM(X, Y, Knew, C, alphaInit) ;
%       %  GradNew = gradRMKLPlus(Knew, pos,Alpha_sv) ;
%    
NormGrad    = currentGrad'*currentGrad;
GradNew     = currentGrad/sqrt(NormGrad);
CostOld     = currentCost;
CostNew     = currentCost;

% if (NormGrad < option.numericalprecision )
%     mu = muNew;
%     return;
% end
%---------------------------------------------------------------
% Compute reduced Gradient and descent direction
%%--------------------------------------------------------------

switch option.firstbasevariable
    case 'first'
        [val,coord] = max(muNew) ;

    case 'random'
        [val,coord] = max(muNew) ;
        coord       = find(muNew == val);
        indperm     = randperm(length(coord));
        coord       = coord(indperm(1));
    case 'fullrandom'
        indzero     = find(muNew ~= 0);
        if ~isempty(indzero)
            [mini,coord]= min(GradNew(indzero));
            coord       = indzero(coord);
        else
            [val,coord] = max(muNew) ;
        end;
        
end;
GradNew = GradNew - GradNew(coord) ;
desc    = - GradNew.* ( (muNew>0) | (GradNew<0) ) ;  % desc - descent direction
desc(coord) = - sum(desc);  % NB:  GradNew(coord) = 0



% mu is variable mu in the paper J(mu)
%----------------------------------------------------
% Compute optimal stepsize
%-----------------------------------------------------
stepmin  = 0;
costmin  = CostOld ;
costmax  = 0 ;
%-----------------------------------------------------
% maximum stepsize
%-----------------------------------------------------
ind     = find(desc<0);
stepmax = min(-(muNew(ind))./desc(ind));
deltmax = stepmax;
if isempty(stepmax) | stepmax==0
    mu = muNew ;
    return
end,
if stepmax > 0.1
     stepmax=0.1;
end;

%-----------------------------------------------------
%  Projected gradient
%-----------------------------------------------------

while costmax < costmin;
    alphaInit      = zeros(size(Y)); 

    alphaInit(pos) = Alpha_sv .* Y(pos); 
    mu_tmp = muNew + stepmax * desc;  % desc - descent direction
    % kernel matrix with a fixed value of mu
    kernel = combinedKernels(basicKernels, mu_tmp);

    [Alp_aux, Alp_sv_aux, costmax, w_sqd,  bAux, posaux, train_err] = SVM([], Y, kernel, option.lambdareg, C, alphaInit);      
    [r beta] = radius_by_LP(componentDistances, mu_tmp); 

    %[1/w_sqd costmax r  r*w_sqd]
    costmax = costmax + C_r/2*r;
    if costmax < costmin
        costmin = costmax;
        muNew   = muNew + stepmax * desc;
            
        % project descent direction in the new admissible cone
        % keep the same direction of descent while cost decrease
        %desc = desc .* ( (muNew>0) | (desc>0) ) ;
        desc = desc .* ( (muNew > option.numericalprecision) | (desc>0) ) ;
        desc(coord) = - sum(desc([[1:coord-1] [coord+1:end]]));  
        ind     = find(desc < 0);
        Alpha_sv  = Alp_sv_aux;
        b       = bAux;
        pos     = posaux;
        if ~isempty(ind)
            stepmax = min(-(muNew(ind))./desc(ind));
            deltmax = stepmax;
            costmax = 0;
        else
            stepmax = 0;
            deltmax = 0;
        end;
        
    end;
end;


%-----------------------------------------------------
%  Linesearch
%-----------------------------------------------------

Step = [stepmin stepmax];
Cost = [costmin costmax];
[val,coord] = min(Cost);
% optimization of stepsize by golden search
while (stepmax - stepmin) > option.goldensearch_deltmax*(abs(deltmax))  & stepmax > eps;
    stepmedr = stepmin+(stepmax-stepmin)/gold;
    stepmedl = stepmin+(stepmedr-stepmin)/gold;                     
    %'svmclass'
    alphaInit      = zeros(size(Y));
    alphaInit(pos) = Alpha_sv .* Y(pos); 
    mu_r  =  muNew  + stepmedr * desc;  % desc - descent direction
    mu_l  =  muNew  + stepmedl * desc;
    % kernel matrix with a fixed value of mu_r
    kernel = combinedKernels(basicKernels, mu_r);
    [Alpr, Alpsupr, costmedr, w_sqd, br, posr, train_err] = SVM([], Y, kernel,option.lambdareg, C, alphaInit);
    [r beta] = radius_by_LP(componentDistances, mu_r); 

    %[1/w_sqd costmedr r  r*w_sqd]
    
    costmedr = costmedr + C_r/2*r;
    
    alphaInit       = zeros(size(Y));
    alphaInit(posr) = Alpsupr .* Y(posr); 
    % kernel matrix with a fixed value of mu_l
    kernel = combinedKernels(basicKernels, mu_l);    
    [Alpl, Alpsupl, costmedl, w_sqd, bl, posl, train_err] = SVM([], Y, kernel, option.lambdareg, C, alphaInit);       
  
    [r beta] = radius_by_LP(componentDistances, mu_l); 

    %[1/w_sqd costmedl r  r*w_sqd]
    costmedl = costmedl + C_r/2*r;
     
    Step = [stepmin stepmedl stepmedr stepmax];
    Cost = [costmin costmedl costmedr costmax];
    [val,coord] = min(Cost);
    switch coord
        case 1
            stepmax = stepmedl;
            costmax = costmedl;
            pos     = posl;
            Alpha_sv  = Alpsupl;
            b       = bl;
        case 2
            stepmax = stepmedr;
            costmax = costmedr;
            pos     = posr;
            Alpha_sv  = Alpsupr;
            b       = br;
        case 3
            stepmin = stepmedl;
            costmin = costmedl;
            pos     = posl;
            Alpha_sv  = Alpsupl;
            b       = bl;
        case 4
            stepmin = stepmedr;
            costmin = costmedr;
            pos     = posr;
            Alpha_sv  = Alpsupr;
            b       = br;
    end;
end;


%---------------------------------
% Final Updates
%---------------------------------

CostNew = Cost(coord) ;
step = Step(coord) ;
% mu update
if CostNew < CostOld ;
    muNew = muNew + step * desc;  
    
end;       

mu = muNew ;
