function [beta, Alpha_sv,b,pos,CostNew] = RSVMPlusl1_withoutMu_update(basicKernels, currentBeta, pos, Alpha_sv,b,C, C_r,Y, currentGrad, currentCost, compDist_vect_form, option, largeScale)

%%
% Extend from Simple MKL - Bach 2009
% Copyright Huyen Do 2011
%%

%------------------------------------------------------------------------------%
% Initialize
%------------------------------------------------------------------------------%
% %
gold    = (sqrt(5) + 1)/2 ;
betaInit  = currentBeta;
betaNew   = betaInit ; 

%---------------------------------------------------------------
% Compute Current Cost and Gradient
%%--------------------------------------------------------------
   
NormGrad    = currentGrad'*currentGrad;
GradNew     = currentGrad/sqrt(NormGrad);
CostOld     = currentCost;
CostNew     = currentCost;

% If the gradient is already close to 0, return.


%---------------------------------------------------------------
% Compute reduced Gradient and descent direction
%%--------------------------------------------------------------

switch option.firstbasevariable
    case 'first'
        [val,coord] = max(betaNew) ;

    case 'random'
        [val,coord] = max(betaNew) ;
        coord       = find(betaNew == val);
        indperm     = randperm(length(coord));
        coord       = coord(indperm(1));
    case 'fullrandom'
        indzero     = find(betaNew ~= 0);
        if ~isempty(indzero)
            [mini,coord]= min(GradNew(indzero));
            coord       = indzero(coord);
        else
            [val,coord] = max(betaNew) ;
        end;
        
end;
GradNew = GradNew - GradNew(coord) ;
desc    =  - GradNew.* ( (betaNew > 0) | (GradNew < 0) ) ;  % desc - descent direction
desc(coord) = - sum(desc);  % NB:  GradNew(coord) = 0

%[sum(betaNew) sum(betaNew'* desc)]

if (NormGrad < option.numericalprecision )
    beta = betaNew ;
    return
end

%% Since we are maximize J(beta), or equivalently minimize -J(beta), we
%% need to change the objective value to -obj
%%


% beta is variable beta in the paper J(beta). We need to maximize J(beta).
%----------------------------------------------------
% Compute optimal stepsize
%-----------------------------------------------------
stepmin  = 0;
costmin  = CostOld ;
costmax  = 0 ;
%-----------------------------------------------------
% maximum stepsize
%-----------------------------------------------------
ind     = find(desc < 0);
stepmax = min(-(betaNew(ind))./desc(ind));
deltmax = stepmax;
if isempty(stepmax) | stepmax==0
    beta = betaNew ;
    return
end

if stepmax > 0.1
     stepmax = 0.1;
end;
 
 alphaInit      = zeros(size(Y));
 alphaInit(pos) = Alpha_sv .* Y(pos);
 beta_tmp = betaNew + stepmax * desc;
 kernel = combinedComponentKernels(basicKernels, beta_tmp, compDist_vect_form,largeScale);
 [alpha, alpha_sv, obj, w_sqd,  b, pos, train_err] = SVM([], Y, kernel, option.lambdareg, C, alphaInit);

 Alpha_sv = alpha_sv; 
 CostNew = -obj;
 beta = beta_tmp; 
 
return %%% faster!!!!!!!!!¨

%-----------------------------------------------------
%  Projected gradient
%-----------------------------------------------------

while costmax < costmin;
    % 'svmclass'
    alphaInit      = zeros(size(Y)); 

    alphaInit(pos) = Alpha_sv .* Y(pos); 
    %alphaInit = [];
    beta_tmp = betaNew + stepmax * desc;  % desc - desceasing direction
    % kernel matrix with a fixed value of beta
    kernel = combinedComponentKernels(basicKernels, beta_tmp, compDist_vect_form,largeScale);

    [Alp_aux, Alp_sv_aux, obj, w_sqd,  bAux, posaux, train_err] = SVM([], Y, kernel, option.lambdareg, C, alphaInit);      
   
    
    costmax = -obj;
    if costmax < costmin
        costmin = costmax;
        betaNew   = betaNew + stepmax * desc;
            
        % project descent direction in the new admissible cone
        % keep the same direction of descent while cost decrease
        %desc = desc .* ( (betaNew>0) | (desc>0) ) ;
        desc = desc .* ( (betaNew > option.numericalprecision) | (desc > 0) ) ;
        desc(coord) = - sum(desc([[1:coord-1] [coord+1:end]]));  
        ind     = find(desc < 0);
        Alpha_sv  = Alp_sv_aux;
        b       = bAux;
        pos     = posaux;
        if ~isempty(ind)
            stepmax = min(-(betaNew(ind))./desc(ind));
            deltmax = stepmax;
            costmax = 0;
        else
            stepmax = 0;
            deltmax = 0;
        end;
        
    end;
end;


%-----------------------------------------------------
%  Linesearch
%-----------------------------------------------------

Step = [stepmin stepmax];
Cost = [costmin costmax];
[val,coord] = min(Cost);
% optimization of stepsize by golden search
while (stepmax - stepmin) > option.goldensearch_deltmax*(abs(deltmax))  & stepmax > eps;
    stepmedr = stepmin+(stepmax-stepmin)/gold;
    stepmedl = stepmin+(stepmedr-stepmin)/gold;                     
    %'svmclass'
    alphaInit      = zeros(size(Y));
    alphaInit(pos) = Alpha_sv .* Y(pos); 
    %alphaInit = [];
    beta_r  =  betaNew  + stepmedr * desc;  % desc - descent direction
    beta_l  =  betaNew  + stepmedl * desc;
    
    % kernel matrix with a fixed value of beta_r
    kernel = combinedComponentKernels(basicKernels, beta_r, compDist_vect_form,largeScale);
    
    [Alpr, Alpsupr, objr, w_sqd, br, posr, train_err] = SVM([], Y, kernel,option.lambdareg, C, alphaInit);
   
    costmedr = -objr;
       
    alphaInit       = zeros(size(Y));
    alphaInit(posr) = Alpsupr .* Y(posr); 
    %alphaInit = [];
    % kernel matrix with a fixed value of beta_l
    kernel = combinedComponentKernels(basicKernels, beta_l,compDist_vect_form,largeScale);  
    
    [Alpl, Alpsupl, objl, w_sqd, bl, posl, train_err] = SVM([], Y, kernel, option.lambdareg, C, alphaInit);       
  
    costmedl = -objl;
     
    Step = [stepmin stepmedl stepmedr stepmax];
    Cost = [costmin costmedl costmedr costmax];
    [val,coord] = min(Cost);
    switch coord
        case 1
            stepmax = stepmedl;
            costmax = costmedl;
            pos     = posl;
            Alpha_sv  = Alpsupl;
            b       = bl;
        case 2
            stepmax = stepmedr;
            costmax = costmedr;
            pos     = posr;
            Alpha_sv  = Alpsupr;
            b       = br;
        case 3
            stepmin = stepmedl;
            costmin = costmedl;
            pos     = posl;
            Alpha_sv  = Alpsupl;
            b       = bl;
        case 4
            stepmin = stepmedr;
            costmin = costmedr;
            pos     = posr;
            Alpha_sv  = Alpsupr;
            b       = br;
    end;
end;


%---------------------------------
% Final Updates
%---------------------------------

CostNew = Cost(coord) ;
step = Step(coord) ;
% beta update
if CostNew < CostOld ;
    betaNew = betaNew + step * desc;  
    
end;       

beta = betaNew ;

