function [alpha, alpha_sv, obj_value, w_sqd, b, pos, train_err] = SVM(X, Y, kernel, lambdareg, C, alphaInit) %SVM(X,Y, kernel, C)

%% Standard L_1 SVM 
% alpha: full alpha - lagrange variable including zero value (support
% vector)
% alpha_sv: alpha(sv) .* y(sv): support lagrange variable with the label.
% The classifier will be  y(x) = K(x, sv) * alpha_sv + b
% lambdareg: regularize lambda for monpq functions

% Copyright: Huyen Do 2010-2011
%%

if nargin< 5
    alphaInit = [];
end
H = kernel  .* (Y*Y') ;   % k_{ij} = x_i x_j y_i y_j;


%% original SVM - the standard SVM with L1 soft margin or hard margin
 
%  % original problem  - standard SVM
f =  ones(length(Y),1);
% 
%  cvx_begin
%  cvx_quiet true
%      variable alpha(length(Y))
%      minimize ((1/2)*quad_form(alpha,H) - f'*alpha );
%      subject to
%         alpha >= 0;  
%         alpha <= C; 
%         Y' * alpha == 0;
%          
%  cvx_end
%  obj_value = cvx_optval; 
% 
% obj_value

%% use quadprog of matlab
% % x = quadprog(H,f,A,b,Aeq,beq,lb,ub) returns a vector x that minimizes 1/2*x'*H*x + f'*x 
% % subject to A*x ≤ b and Aeq*x = beq, also defines a set of lower and upper bounds 
% % on the design variables, x, so that the solution is in the range lb ≤ x ≤ ub. 
% % If no equalities exist, set Aeq = [] and beq = [].
%%
% % %'
% % options = optimset('MaxIter',100000);
% % 
% % A = [];
% % b = [];
% % Aeq = Y';
% % beq = 0;
% % lb = zeros(size(Y));
% % ub = C * ones(size(Y)); 
% % 
% % [alpha obj_value] = quadprog(H, -f, A, b, Aeq, beq,lb, ub, 0.1*ones(length(Y),1), options);
% % obj_value
% %  

% %  % for non linear kernel, w = \sum alpha_i y_i \phi(x)_i
% %  
% % svIndex = find(alpha > 0.000001 & alpha < C - 0.000001);
% % % if (isempty(svIndex))
% % %     svIndex = find(alpha > 0 & alpha < C);
% % % end
% % 
% % % the hyperplane: wx + b = (alpha .*Y)' kernel(:, x) + b = kernel(x,:)* (alpha.*Y)
% % 
% % b = Y(svIndex(1)) -  kernel(svIndex(1),:) * (alpha .*Y);
% %  
% % pos = find(alpha >0.000000001);
% % alpha_sv = alpha(pos);

%% use monqp

Aeq = Y;

c = C * ones(size(Y)); 

if isinf(C)        
    [alphasv , b , pos] =  monqpCinfty2(H,f,Aeq,0,lambdareg,0,alphaInit);  
else      
    [alphasv , b , pos] =  monqp2(H,f,Aeq,0,c,lambdareg,0,alphaInit);         
    
end


alpha = zeros(length(Y),1);
alpha(pos) = alphasv;

alpha_sv = alphasv .* Y(pos);

obj_value = - 0.5*alpha'*H*alpha + f'*alpha;


% 1/margin: w squared:
w_sqd = alpha' * H * alpha;

% training error 
 
distance_to_margin = kernel * (alpha .*Y)  + b; 

predict = sign(Y .* distance_to_margin); 
train_err = sum(predict < 0) / length(Y);


%% testing: compare the margin squared value and the real radius squared
% value. Normally margin < radius, but in case all instances lie on 1 side
% of the hyperplan, this inequality may not hold. 
% 
% margin_squared = 1/ w_sqd; 
% r = componentRadiusSquared(X, kernel); 
% fprintf('SVM margin squared - %f - trainErr %f - obj %f - distToMargin %g %g - realRadius %f \n', margin_squared, train_err, obj_value , [sum((distance_to_margin >= 0))  sum((distance_to_margin < 0))], r);

%%  testing: if b the same for all svIndex 
% svIndex  = find(alpha > 0 & alpha < C);
% b = Y(svIndex(1)) -  kernel(svIndex(1),:) * ((alpha) .*Y)
% b_all = Y(svIndex) -  kernel(svIndex,:) * ((alpha) .*Y)
% Y(svIndex) .* (kernel(svIndex,:) * ((alpha) .*Y)  + b)
