function  [alphaall, alpha_sv, obj, w_sqd, b, pos, train_err] = SVM_L2soft(X, Y, kernel, lambdareg, C, alphaInit)

%%
% Implement SVM with L2 soft margin
% alpha: full alpha - lagrange variable including zero value (support
% vector)
% alpha_sv: alpha(sv) .* y(sv): support lagrange variable with the label.
% The classifier y(x) = K(x, sv) * alpha_sv +  b
% Copyright Huyen Do 2011
%%

if nargin < 5
    alphaInit = [];
end

% Use L2 soft margin, so we have to adjust the kernel by C
numInstances = length(Y);
delta = eye(numInstances, numInstances);
delta = delta * 1/C;
kernel = kernel + delta;

H = kernel  .* (Y*Y') ;   % k_{ij} = x_i x_j y_i y_j;

f = ones(numInstances,1);


%% Use function in SimpleMKL: monpq, monpqCinfty
%----------------------------------------------------------------------
%      monqp(H,c,A,b,C) solves the quadratic programming problem:
% 
%    min 0.5*x'Hx - c'x   subject to:  A'x = b  and  0 <= x <= C 
%     x    
%----------------------------------------------------------------------
% Here we use L2 soft margin, so C = Inf 

b = 0;                                                     
[alpha_sv, b, pos] = monqpCinfty2(H,f,Y,b,lambdareg, 0, alphaInit);         

%alpha: only nonzero value is stored , positions of these values are stored in pos
alphaall        = zeros(length(Y),1);
alphaall(pos)   = alpha_sv;
obj = - 0.5*alphaall'*H*alphaall +f'*alphaall;


ysup = Y(pos);
alpha_sv = alpha_sv.*ysup;

% 1/margin: w squared:
w_sqd = sum(alpha_sv)  - 1/C * (alphaall' * alphaall);


% training error 
 
predict = sign(Y .* (kernel(:,pos) * (alpha_sv .*Y(pos))  + b)); 
train_err = length(find(predict < 0)) /length(Y);

