function [res ] =  innerCV_RSVMPlusl1(Xtrain, Ytrain, Xtest, Ytest,  kfold_inner, vectorC, vectorC_r, randomSeed)
%% Inner kfold_innner cross validation  to choose the optimal hyperparams.
%% X, Y, K are in fact the X,Y,k(outerTrain)

% Initialize
lambdareg = 1e-10;
res.kfold_inner = kfold_inner;
res.lambreg  = lambdareg; 
res.vectorC = vectorC; 
res.vectorC_r = vectorC_r; 
res.randomSeed = randomSeed; 

option.lambdareg = lambdareg; 
option.verbose = 1; 

rand('twister',randomSeed);
randOrder = randperm(length(Ytrain));

numFeatures = size(Xtrain,2); 

Ktrain = zeros(length(Ytrain), length(Ytrain), numFeatures );
for k = 1:numFeatures
    Ktrain(:,:,k) = Xtrain(:, k) * Xtrain(:, k)'; 
end

% 
% kernelTooBig = false;
% try
%     
%     Ktest = zeros(length(Ytest), length(Ytrain), numFeatures);
%     for k = 1:numFeatures
%         Ktest(:,:,k) = Xtest(:, k) * Xtrain(:, k)';
%     end
% catch
%     
%     kernelTooBig = True;
%     
% end



outerTrain = 1:length(Ytrain); 
outerTrain = outerTrain(randOrder);

numInnerTest = floor(length(outerTrain)/kfold_inner);

YPredict_inner_err= zeros(length(outerTrain),length(vectorC), length(vectorC_r));

%
%% Inner cross validation to choose the optimal hyperparameters
tic; 

for i = 1: kfold_inner
    fprintf('RSVMPlusl1 Cross validation.  insideCV =  %g \n', i);
    %index of inner.test and inner.training samples
    index = [];
    if(i < kfold_inner)
        index = ((i-1)*numInnerTest+1):(i*numInnerTest);
        
    else
        index = ((i-1)*numInnerTest+1):length(outerTrain);
        
    end
    innerTest  = outerTrain(index);
    innerTrain = setdiff(outerTrain, innerTest);
    % Compute component squared distances
    [compDistance_vect_form componentDist]=  componentDistances(Xtrain(innerTrain,:));
    
    % Repeat for different value of C
    for j =  1:length(vectorC)
        C = vectorC(j);
        % Repeat for different value of C_r
        for k = 1:length(vectorC_r)
            C_r = vectorC_r(k);
            
            % RSVMPlus_constrainedmu  ==============================
            [mu w  b posw history obj] = RSVMPlusl1(Xtrain(innerTrain,:), Ytrain(innerTrain), Ktrain(innerTrain, innerTrain,:), compDistance_vect_form, C,C_r, option);
            
            Ktest_tmp       = combinedKernels(Ktrain, mu);
            y_predict       = Ktest_tmp(innerTest,innerTrain(posw))  * w  + b;
            %accuracy    = mean(sign(y_predict) == Ytrain(innerTest));
            
            YPredict_inner_err(innerTest,j,k) = (sign(y_predict) ~= Ytrain(innerTest));
            
%             inner.error(i,j,k)  = 1 - accuracy;
%             inner.margin(i,j,k) = w' * w;
%             inner.obj(i,j,k)    = obj;
            % end RSVMPlus_constrainedmu =============================
        end
    end
end; % end inside cross validation
 
inner_err = mean(YPredict_inner_err); % mean error for each C and C_r

min_inner_err = min(min(inner_err));
indC_Cr = find(inner_err == min_inner_err);

optC_Cr = cell(length(indC_Cr),1);
for k  = 1:length(indC_Cr)
    i = ceil(indC_Cr(k)/length(vectorC));
    j = indC_Cr(k) - (i - 1) * length(vectorC);
    optC_Cr{k} = [vectorC(j) vectorC_r(i)];
    
end
    

% Compute component squared distances
[compDistance_vect_form2 componentDist]=  componentDistances(Xtrain);

% Now compute with all optimal pair C and C_r. Choose the best accuracy
% RSVMPlus_constrainedmu==============================

C       = optC_Cr{1}(1);
C_r     = optC_Cr{1}(2);

[mu w  b posw history obj] = RSVMPlusl1(Xtrain, Ytrain, Ktrain, compDistance_vect_form2, C, C_r, option);
%if (kernelTooBig)
Ktest = zeros(length(Ytest), length(posw), numFeatures);
for k = 1:numFeatures
    Ktest(:,:,k) = Xtest(:, k) * Xtrain(posw, k)';
end
Ktest_tmp     = combinedKernels(Ktest, mu);
% else
%     Ktest_tmp     = combinedKernels(Ktest, mu);
% end

y_predict = Ktest_tmp  * w  + b;
accuracy  = mean(sign(y_predict) == Ytest);

C_opt = C;
C_r_opt = C_r;
for i = 2:length(optC_Cr)
    
    C       = optC_Cr{i}(1);
    C_r     = optC_Cr{i}(2);
    
    [mu_tmp w_tmp  b_tmp posw_tmp history_tmp obj_tmp] = RSVMPlusl1(Xtrain, Ytrain, Ktrain, compDistance_vect_form2, C, C_r, option);
    
    Ktest_tmp = zeros(length(Ytest), length(posw_tmp), numFeatures);
    for k = 1:numFeatures
        Ktest_tmp(:,:,k) = Xtest(:, k) * Xtrain(posw_tmp, k)';
    end

    Ktest_tmp     = combinedKernels(Ktest_tmp, mu);
    y_predict_tmp = Ktest_tmp  * w_tmp  + b_tmp;
    
    accuracy_tmp  = mean(sign(y_predict_tmp) == Ytest);
    if(accuracy_tmp > accuracy)
        accuracy    = accuracy_tmp;
        mu          = mu_tmp;
        w           = w_tmp;
        b           = b_tmp;
        posw        = posw_tmp;
        obj         = obj_tmp;
        y_predict   = y_predict_tmp;
        C_opt       = C;
        C_r_opt     = C_r;
    end
    
end
fprintf('RSVMPlusl1 Cross validation. optimalC = %g, optC_r = %g, error %f\n', C_opt, C_r_opt, 1-accuracy );

%End RSVMPlus_constrainedmul1
res.time = toc; 

classifier.error_test =   1 -accuracy;
classifier.y_predict  =  y_predict;
classifier.mu         = mu;
classifier.w          = w;
classifier.b          = b;
classifier.posw       = posw;
classifier.obj        = obj; 

res.classifier       = classifier;
res.Ytest            = Ytest;
res.YPrediction      = sign(y_predict); 
res.optimalC         = C_opt;
res.optimalCr        = C_r_opt;
res.allOptC_Cr       = optC_Cr;
res.numNonZeroKernel = sum(abs(mu) > 0.0001);

res.averageErr       =  sum(res.YPrediction  ~= Ytest)/length(Ytest);

fprintf('C_optimal = %g , error = %f \n',  C_opt,  classifier.error_test )
res.averageErr



