function [res] =  innerCV_SVMl1(Xtrain, Ytrain, Xtest, Ytest, Ktrain, Ktest,  kfold_inner, vectorC, randomSeed)
%% SVM - with L1 regularization of slack valiables - L1 soft margin SVM
% This function compute the error of one random split of data. 
%% Params
% Input: 
% 
% Xtrain: training input X, rows are instances and columns are features. 
% Ytrain: training labels. 
% Xtest:  testing intances
% Ytest:  testing labels.
% Ktrain: kernel of (Xtrain, Xtrain)
% Ktest:  kernel of (Xtest, Xtrain)
% kfold_inner: fold number of inner cross validation to choose the optimal hyperparameters. 
%              We use kfold_inner = 5 in our second set of experiments.
% vectorC: set of C values 
%              We choose the optimal C from set vectorC=[0.1 1 10 100 1000]
%              using inner 5-fold cross validation in our experiments
% randomSeed: the random seed to permute the instances to create inner fold
%             for inner cross validation (to choose optimal hyperparams)
% Output:
% res:   a structure, including error of the data split, the value of y
%        prediction, and other values. 
%
%% Copyright@Huyen Do 2012. 

% Initialize
lambdareg = 1e-10;
res.kfold_inner = kfold_inner;
res.vectorC = vectorC; 
res.randomSeed = randomSeed; 


rand('twister',randomSeed);
randOrder = randperm(length(Ytrain));

outerTrain = 1:length(Ytrain); 
outerTrain = outerTrain(randOrder);

numInnerTest = floor(length(outerTrain)/kfold_inner);

YPredict_inner_err = zeros(length(outerTrain),length(vectorC));

%
%% Inner cross validation to choose the optimal hyperparameters
tic;  % time counting

for i = 1: kfold_inner
    fprintf('SVMl1 Inner Cross validation.  insideCV =  %g \n', i);
    %index of inner.test and inner.training samples
    index = [];
    if(i < kfold_inner)
        index = ((i-1)*numInnerTest+1):(i*numInnerTest);
        
    else
        index = ((i-1)*numInnerTest+1):length(outerTrain);
        
    end
    innerTest  = outerTrain(index);
    innerTrain = setdiff(outerTrain, innerTest);
    
    % Repeat for different value of C
    for j =  1:length(vectorC)
        C_alp = vectorC(j);
        
        %Begin L1 norm SVM
        [alpha, alpha_sv, obj_value, w_sqd, b, pos, train_err] = SVM(Xtrain(innerTrain,:), Ytrain(innerTrain), Ktrain(innerTrain,innerTrain), lambdareg, C_alp,[]);
        
        y_predict = Ktrain(innerTest,innerTrain(pos))  * alpha_sv  + b;
        
        YPredict_inner_err(innerTest,j) = (sign(y_predict) ~= Ytrain(innerTest));
        
        fprintf('SVMl1 Inner Cross validation.  insideCV =  %g, error %f\n', i, sum(YPredict_inner_err(innerTest,j))/ length(Ytrain(innerTest)));
        
    end %End SVM
end; % end inner cross validation
inner_err = mean(YPredict_inner_err);  % mean by column or mean by each C
[optimalC optimalCIndex] = find(inner_err == min(inner_err));

% Now we have optimal C from previous cross validation procedure. Train again with the optimal C
% Then test with instances in outerTest

%Begin SVM l1 
C =  vectorC(optimalCIndex(1));

[alpha, alpha_sv, obj_value, w_sqrd, b, pos, train_err] = SVM(Xtrain, Ytrain, Ktrain, lambdareg, C,[]);

y_predict = Ktest(:,pos)  * alpha_sv   + b;

accuracy = mean(sign(y_predict) == Ytest);

C_opt = C;


res.time = toc; 

classifier.w_sqrd   = w_sqrd;
classifier.alpha    = alpha;
classifier.error_test   = 1-accuracy;
classifier.y_predict    = y_predict;
classifier.obj_value    = obj_value;
classifier.b            = b;

res.classifier       = classifier;
res.Ytest            = Ytest;
res.optimalC         = C_opt;
res.allOptC          = vectorC(optimalCIndex);
res.YPrediction      = sign(y_predict); 
res.averageErr       =  sum(res.YPrediction  ~= Ytest)/length(Ytest);

fprintf('C_optimal = %g , error = %f \n',  C_opt,  classifier.error_test )
res.averageErr

